<?php

namespace Eform\Business\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Eform\Business\Events\BusinessWorkflow as BusinessWorkflowEvent;
use Eform\Business\Notifications\BusinessWorkflow as BusinessWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class BusinessWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(BusinessWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $business is submitted.
     *
     * @param BusinessWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(BusinessWorkflowEvent $event)
    {
        $client = $event->business->client;
        Notification::send($client, new BusinessWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $business is published.
     *
     * @param BusinessWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(BusinessWorkflowEvent $event)
    {
        $client = $event->business->client;
        Notification::send($client, new BusinessWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $business is approved.
     *
     * @param BusinessWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(BusinessWorkflowEvent $event)
    {
        $client = $event->business->client;
        Notification::send($client, new BusinessWorkflowNotification($event));
    }

    /**
     * Handles the $business workflow event as a listener.
     *
     * @param BusinessWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(BusinessWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
